@@ -4,7 +4,7 @@
  * Filename:      superio.c
  * Description:   kernel module that configures the PC97338
  *                Super I/O chip present in some ThinkPads
- * Author:        Thomas Hood <jdthood@mail.com>
+ * Author:        Thomas Hood
  * Reference:     National Semiconductor PC87338/PC97338 data sheet.  November, 1998.
  * Created:       19 July 1999 
  *
@@ -28,62 +28,40 @@
  *     
  ********************************************************************/
 
-#include "thinkpad_mod_defines.h"
+#include "thinkpad_driver.h"
 
 #include <linux/kernel.h>
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/ioport.h>
-#ifdef USE_PROC_FS
 #include <linux/proc_fs.h>
-#endif
+#include <linux/version.h>
 #include <asm/uaccess.h>
 #include <asm/io.h>
 #include "thinkpad_common.h"
 #include "superio.h"
 
 
-/****** defines ******/
-
-#define SZ_SUPERIO_MODULE_VERSION "1.0"
-#define SZ_SUPERIO_MODULE_NAME "superio"
+/****** definitions ******/
 
 
 /****** declarations ******/
 
-#ifdef USE_PROC_FS
-#ifndef NEW_PROC
-extern struct proc_dir_entry _proc_dir_entryThinkpadDir;
-static int superio_read_proc(
-	char *pchBuf,
-	char **ppchStart,
-	off_t offThe,
-	int intLen,
-	int intUnused
-);
-#endif
-#endif
+extern struct proc_dir_entry *thinkpad_ppde;
 
 
 /****** variables ******/
-static char _szSuperioName[] = SZ_SUPERIO_MODULE_NAME;
-static char _szSuperioVersion[] = SZ_SUPERIO_MODULE_VERSION;
+static const char _szMyName[] = "superio";
+static const char _szImName[] = "superio_do";
+static const char _szMyVersion[] = "5.0";
+static const char _szProcfile[] = "driver/thinkpad/superio";
 static word _wPortIndex, _wPortData;
-static flag_t _fSuperioReady = 0;
+static struct resource *_presourceSuperio;
 
-#ifdef USE_PROC_FS
-#ifndef NEW_PROC
-static flag_t _fCreatedProcSuperio = 0;
-static struct proc_dir_entry _proc_dir_entrySuperio = {
-	0,                  /* low_ino: the inode--dynamic */
-	7, "superio",       /* len of name, name */
-	S_IFREG | S_IRUGO,  /* mode */
-	1, 0, 0,            /* nlinks, owner, group */
-	0,                  /* size--unused */
-	NULL,               /* point to operations--use default */
-	&superio_read_proc
-};
-#endif
+#ifdef MODULE
+MODULE_AUTHOR( "Thomas Hood" );
+MODULE_DESCRIPTION( "Driver for the Super I/O chip on IBM ThinkPads" );
+MODULE_LICENSE( "GPL" );
 #endif
 
 
@@ -91,16 +69,18 @@ static struct proc_dir_entry _proc_dir_entrySuperio = {
 
 /*
  * We use _p variant of outb just for safety
+ * getregb and putregb use the global variables 
+ * _wPortIndex and _wPortData
  */
 static byte getregb( byte bWhere )
 {
 	byte bGot;
 	unsigned long flags;
 
-	save_flags( flags ); cli();
+	local_irq_save( flags );
 	outb_p( bWhere, _wPortIndex );
 	bGot = inb( _wPortData );
-	restore_flags( flags );
+	local_irq_restore( flags );
 
 	return bGot;
 }
@@ -110,10 +90,10 @@ static void putregb( byte bWhat, byte bWhere )
 {
 	unsigned long flags;
 
-	save_flags( flags ); cli();
+	local_irq_save( flags );
 	outb_p( bWhere, _wPortIndex );
 	outb( bWhat, _wPortData );
-	restore_flags( flags );
+	local_irq_restore( flags );
 
 	return;
 }
@@ -210,10 +190,17 @@ static int get_pnp_regs( dword *pdwRegs )
 }
 
 
-static flag_t is_in_pnp_mode( word wPortIndex )
+static flag_t is_in_pnp_mode( word wPortIndex, word wPortData )
 {
+	byte bGot;
+	unsigned long flags;
 
-	return (flag_t)( (getregb(0x1b) >> 3) & 1 );
+	local_irq_save( flags );
+	outb_p( 0x1b, wPortIndex );
+	bGot = inb( wPortData );
+	local_irq_restore( flags );
+
+	return (flag_t)( (bGot >> 3) & 1 );
 }
 
 
@@ -480,10 +467,9 @@ static int get_par_mode( superio_par_mode_t *pmode )
 		return 0;
 	}
 
-	printk(
-		KERN_INFO
-		"%s: parallel port mode not recognized: PCR = 0x%x  PTR = 0x%x\n",
-		_szSuperioName,
+	printk(KERN_INFO
+		"%s: Parallel port mode not recognized: PCR = 0x%x  PTR = 0x%x\n",
+		_szMyName,
 		bPCR, bPTR
 	);
 	*pmode = SUPERIO_PAR_MODE_UNRECOGNIZED;
@@ -651,36 +637,21 @@ static int set_ser_base_address(
 }
 
 
-#ifdef USE_PROC_FS
 static int superio_read_proc(
 	char *pchBuf, 
 	char **ppchStart, 
-	off_t offThe,
-	int intLen, 
-#ifdef NEW_PROC
-	int *eof, 
+	off_t off,
+	int intCount, 
+	int *pintEof, 
 	void *data
-#else
-	int intUnused
-#endif
 ) {
-
-	if ( !_fSuperioReady ) {
-		return sprintf(
-			pchBuf,
-			"%s version %s not ready\n",
-			_szSuperioName, _szSuperioVersion
-		);
-	}
-
-	return sprintf(
-		pchBuf,
+	return snprintf(
+		pchBuf, intCount,
 		"%s version %s controlling Super I/O chip at ioports 0x%x, 0x%x\n",
-		_szSuperioName, _szSuperioVersion,
+		_szMyName, _szMyVersion,
 		_wPortIndex, _wPortData
 	);
 }
-#endif
 
 static int superio_do_func(
 	superio_inparm_t *pinparmThe,
@@ -701,13 +672,12 @@ static int superio_do_func(
 		case SUPERIO_FUNC_FDD_BASE_GET:
 			return get_fdd_base_address( (word *)&(poutparmThe->dwParm1) );
 		case SUPERIO_FUNC_PAR_IRQ_GET:
-			if ( is_in_pnp_mode( _wPortIndex ) ) {
+			if ( is_in_pnp_mode( _wPortIndex, _wPortData ) ) {
 				return get_par_irq_pnpmode( (byte *)&(poutparmThe->dwParm1) );
 			} else {
-				printk(
-					KERN_INFO
+				printk(KERN_INFO
 					"%s: Super I/O chip is in legacy mode!\n",
-					_szSuperioName
+					_szMyName
 				);
 				return get_par_irq_legacymode( (byte *)&(poutparmThe->dwParm1) );
 			}
@@ -757,9 +727,6 @@ int superio_do(
 	unsigned long ulRtnCopy;
 	int intRtnDo;
 
-	if ( !_fSuperioReady )
-		return -ETHINKPAD_EXECUTION;
-
 	ulRtnCopy = copy_from_user(
 		&inparmThe,
 		(byte *)&(((superio_ioparm_t *)ulongIoctlArg)->in),
@@ -770,10 +737,9 @@ int superio_do(
 	intRtnDo = superio_do_func( &inparmThe, &outparmThe, fCallerHasWritePerm );
 
 	if ( intRtnDo == -EACCES ) {
-		printk(
-			KERN_ERR
+		printk(KERN_ERR
 			"%s: caller does not have permission to do this to the Super IO device\n",
-			_szSuperioName
+			_szMyName
 		);
 		return intRtnDo;
 	}
@@ -785,7 +751,7 @@ int superio_do(
 	);
 	if ( ulRtnCopy ) return -EFAULT;
 
-	return intRtnDo;
+	return intRtnDo; /* Should be either 0 or -EACCES or -EINVAL */
 }
 
 
@@ -799,7 +765,7 @@ static flag_t a_super_io_chip_appears_to_be_at( word wPort )
 	byte bSave, bID;
 	unsigned long flags;
 
-	save_flags( flags ); cli();
+	local_irq_save( flags );
 
 	bSave = inb( wPort );
 
@@ -823,12 +789,12 @@ static flag_t a_super_io_chip_appears_to_be_at( word wPort )
 
 /* Yes: */
 	/* No need to restore index port's contents */
-	restore_flags( flags );
+	local_irq_restore( flags );
 	return 1;
 
 No:
 	outb( bSave, wPort ); /* restore the port's contents */
-	restore_flags( flags );
+	local_irq_restore( flags );
 	return 0;
 }
 
@@ -849,120 +815,95 @@ static flag_t see_index_ID( word wPort )
 }
 
 
-int __init init_module( void )
+static int __init superio_init( void )
 {
 	word wPortBase;
 	flag_t fVirgin;
-	int intRtnCheck;
-
-	_fSuperioReady = 0;
 
 	fVirgin = see_index_ID( 0x2e );
 
 	wPortBase = locate_chip();
 
 	if ( wPortBase == 0 ) {
-		printk(
-			KERN_ERR
-			"%s: Super I/O chip not found.  :-(\n",
-			_szSuperioName
+		printk(KERN_ERR
+			"%s: Super I/O chip not found. :-(\n",
+			_szMyName
 		);
 		return -ETHINKPAD_HW_NOT_FOUND;
 	}
+
 	/* chip found at wPortBase */
 
-#if 0
-/* 2000 Oct 4: JDTH: Omit this because the test fails on some newer ThinkPad models */
-	if ( !is_in_pnp_mode( wPortBase ) ) {
-		printk(
-			KERN_ERR
-			"%s: Super I/O chip found at port base 0x%x is not in plug-n-play mode.  :-(\n",
-			_szSuperioName, wPortBase
+	if ( !is_in_pnp_mode( wPortBase, wPortBase+1 ) ) {
+		printk(KERN_ERR
+			"%s: Super I/O chip found at port base 0x%x is not in plug-n-play mode. :-(\n",
+			_szMyName, wPortBase
 		);
 		return -ETHINKPAD_HW_NOT_FOUND;
 	}
-#endif
 
-	printk(
-		KERN_INFO
-		"%s: Super I/O chip (%svirgin) %sin plug-n-play mode found at port base 0x%x.  :-)\n",
-		_szSuperioName,
+	printk(KERN_INFO
+		"%s: Super I/O chip (%svirgin) %sin plug-n-play mode found at port base 0x%x. :-)\n",
+		_szMyName,
 		fVirgin ? "" : "non-",
-		is_in_pnp_mode( wPortBase ) ? "" : "not ",
+		is_in_pnp_mode( wPortBase, wPortBase+1 ) ? "" : "not ",
 		wPortBase
 	);
 
 	/* At this point we assume we have found a working chip at wPortBase */
 
 	/*** request the io ports ***/
-	intRtnCheck = check_region( wPortBase, 2 );
-	if ( intRtnCheck < 0 ) {
-#if 0
+	_presourceSuperio = request_region( wPortBase, 2, "superio" );
+	if ( _presourceSuperio == NULL ) {
 /*
  * The region is claimed by the programmable interrupt controller "pic1"
  * as part of the range 0x20--0x3f.
  */
-		printk(
-			KERN_ERR
-			"%s: the ioports for the Super I/O chip are not available\n",
-			_szSuperioName
+#if 0
+		printk(KERN_ERR
+			"%s: I/O ports for Super I/O chip not available -- aborting.\n",
+			_szMyName
 		);
 		return -ETHINKPAD_HW_NOT_FOUND;
+#else
+		printk(KERN_INFO
+			"%s: I/O ports for Super I/O chip not available, but ignoring this.\n",
+			_szMyName
+		);
 #endif
-	} else {
-		request_region( wPortBase, 2, "superio" );
 	}
 
 	_wPortIndex = wPortBase;
 	_wPortData = wPortBase + 1;
-	_fSuperioReady = 1;
 
-#ifdef USE_PROC_FS
-	/*** Add /proc entry ***/
-#ifdef NEW_PROC
-	if ( !create_proc_read_entry(
-		"thinkpad/superio",
+	if ( !thinkpad_ppde || !create_proc_read_entry(
+		_szProcfile,
 		S_IFREG | S_IRUGO,
 		NULL,
 		superio_read_proc,
 		NULL
 	) ) {
-		printk( KERN_ERR "%s: Could not create_proc_read_entry() /proc/thinkpad/superio\n", _szSuperioName );
+		printk(KERN_ERR "%s: Could not create_proc_read_entry() /proc/%s\n", _szMyName, _szProcfile );
 	}
-#else
-	_fCreatedProcSuperio = 0;
-	if ( proc_register( &_proc_dir_entryThinkpadDir, &_proc_dir_entrySuperio ) ) {
-		printk( KERN_ERR "%s: error returned by proc_register()\n", _szSuperioName );
-	} else {
-		_fCreatedProcSuperio = 1;
-	}
-#endif
-#endif
+	/* proc entry created */
+
+	inter_module_register( _szImName, THIS_MODULE, &superio_do );
 
 	return 0;
 }
 
 
-void cleanup_module( void )
+static void __exit superio_exit( void )
 {
 
-	if ( !_fSuperioReady ) return;
+	inter_module_unregister( _szImName );
 
-	_fSuperioReady = 0;
+ 	remove_proc_entry( _szProcfile, NULL );
 
-#ifdef USE_PROC_FS
-	/*** Remove /proc entry ***/
-#ifdef NEW_PROC
- 	remove_proc_entry("thinkpad/superio", NULL);
-#else
-	if ( _fCreatedProcSuperio ) {
-		int intRtn;
-		_fCreatedProcSuperio = 0;
-		intRtn = proc_unregister( &_proc_dir_entryThinkpadDir, _proc_dir_entrySuperio.low_ino );
-		if ( intRtn ) printk( KERN_ERR "%s: error returned by proc_unregister()\n", _szSuperioName );
-	}
-#endif
-#endif
+	if ( _presourceSuperio != NULL ) release_resource( _presourceSuperio );
 
 	return;
 }
+
+module_init(superio_init);
+module_exit(superio_exit);
